angular.module('myApp.services', [])

// focus
.factory('focus', function($timeout, $window) {
    return function (id) {
        // timeout makes sure that it is invoked after any other event has been triggered.
        // e.g. click events that need to run before the focus or
        // inputs elements that are in a disabled state but are enabled when those events
        // are triggered.
        $timeout(function () {
            var element = $window.document.getElementById(id);
            if (element)
                element.focus();
        });
    };
})

// viewport
.factory('viewport', function() {
    return function () {
        var e = window, a = 'inner';
        if (!('innerWidth' in window)) {
            a = 'client';
            e = document.documentElement || document.body;
        }
        return { width: e[a + 'Width'], height: e[a + 'Height'] }
    }
})

// notificationService
.service('notificationService', function($ionicLoading, $timeout) {
    this.showMessage = function (message, timeout) {
        $ionicLoading.show({
            template: message
        });

        $timeout(function () {
            $ionicLoading.hide();
        }, timeout);
    }
})

// popUpService
.service('popUpService', function($ionicPopup, $timeout) {
    this.showMessage = function(title, message, timeout) {
        var messagePopup = $ionicPopup.show({
            template: '',
            title: title,
            subTitle: message
        });

        $timeout(function () {
            messagePopup.close();
        }, timeout);
    };
})

// shareDataService between controllers
.service('shareDataService', function() {
    var sharedData = [];
    /**
     * Registers an Object to the service
     * @param  {String} key
     * @param  {Object} newObj
     */
    this.registerObj = function (key, newObj) {
        sharedData[key] = newObj;
    };
    /**
     * Returns the requested Object from the service
     * @param  {String} key
     * @return {Object}
     */
    this.getData = function(key) {
        return sharedData[key];
    };
    /**
     * Removes an Object from the service
     * @param  {String} key
     */
    this.removeObject = function(key) {
        var index = sharedData.indexOf(key);
        if (index > -1) {
            array.splice(index, 1);
        }
    }

    this.removeAll = function () {
        sharedData = [];
    }
})

// Data saving service
.service('dataSavingService', function() {
    var savedData = {};
    /**
     * Saves an Object using the window.localStorage
     * @param  {String} key
     * @param  {Object} value
     */
    this.saveItem = function(key, value) {
        window.localStorage.setItem(key, value);
        savedData[key] = value;
    };
    /**
     * Returns the requested item
     * @param  {String} key
     * @return {Object}
     */
    this.loadItem = function(key) {
        var item = window.localStorage.getItem(key);
        savedData[key] = item;
        return item;
    };
    /**
     * Removes an Object from the saved items
     * @param  {String} key
     */
    this.removeItem = function(key) {
        window.localStorage.removeItem(key);
        delete savedData[key];
    };
    /**
     * Clears all saved items
     */
    this.clearAll = function() {
        window.localStorage.clear();
        savedData = {};
    };
})

// Currency Service
.service('currencyService', function($rootScope, $http, dataSavingService) {
    // Initializing variables for the service
         var currencyRates = { EUR: null };
    var actualCurrency = null;
    var base = "EUR";
    var updated = false;
    var currencySymbols = {
        "EUR": '€',
        "AUD": '$',
        "BGN": 'лв',
        "BRL": 'R$',
        "CAD": '$',
        "CHF": 'Fr',
        "CNY": '¥',
        "CZK": 'Kč',
        "DKK": 'kr',
        "GBP": '£',
        "HKD": '$',
        "HRK": 'kn',
        "HUF": 'Ft',
        "IDR": 'Rp',
        "ILS": '₪',
        "INR": '₹',
        "JPY": '¥',
        "KRW": '₩',
        "MXN": '$',
        "MYR": 'RM',
        "NOK": 'kr',
        "NZD": '$',
        "PHP": '₱',
        "PLN": 'zł',
        "RON": 'lei',
        "RUB": 'руб',
        "SEK": 'kr',
        "SGD": '$',
        "THB": '฿',
        "TRY": 'TRY',
        "USD": '$',
        "ZAR": 'R'
    };

    /**
     * Returns the currency symbols
     */
    this.getCurrencySymbols = function() {
        return currencySymbols;
    };

    /**
     * Handles currency conversion based on the fx settings
     * @return {double}
     */
    this.convert = function(number) {
        return fx.convert(number);
    };
    /**
     * Returns the actual currency which is set
     * @return {Object}
     */
    this.getActualCurrency = function() {
        var response = [];
        response['currency'] = actualCurrency;
        if (currencySymbols.hasOwnProperty(actualCurrency)) {
            response['symbol'] = currencySymbols[actualCurrency];
        } else {
            response['symbol'] = actualCurrency;
        }
        return response;
    };
    /**
     * Returns the currency rates
     * @return {Object}
     */
    this.getCurrencyRates = function() {
        if (updated == false) {
            this.updateCurrencyRates();
            $rootScope.$on('updateCurrencyRatesFinished', function (event, args) {
                window.currencyRates = currencyRates;
                return currencyRates;
            });
        } else {
            return currencyRates;
        }
    };
    /**
     * Sets a new currency for the service
     * @param {String} newCurrency
     */
    this.setActualCurrency = function(newCurrency) {
        actualCurrency = newCurrency;
        dataSavingService.saveItem('actualCurrency', newCurrency);
        if (typeof fx !== "undefined") {
            fx.settings = {
                from: base,
                to: actualCurrency
            };
        }
    };
    /**
     * Updates the rates requested from http://api.fixer.io
     * @return {Object}
     */
    this.updateCurrencyRates = function() {
        var url = "http://api.fixer.io/latest?base=" + base;

         var response = {
             data: {
                rates:{'EUR':1}
             }
         }
         
         if (typeof fx !== "undefined" && fx.rates) {
            fx.settings = {
                from: base,
                to: actualCurrency
            };
            fx.rates = currencyRates;
            fx.base = base;
         } else {
         // If not, apply to fxSetup global:
            window.fxSetup = {
                from: base,
                to: actualCurrency,
                rates: currencyRates,
                base: base
            };
         }
         
         //////
       /*
        $http({
            method: 'GET',
            url: url,
            cache: false
        }).then(function (response) {
            response.data.rates[base] = 1;
            currencyRates = response.data.rates;

            // Check if money.js has finished loading:
            if (typeof fx !== "undefined" && fx.rates) {
                fx.settings = {
                    from: base,
                    to: actualCurrency
                };
                fx.rates = currencyRates;
                fx.base = base;
            } else {
                // If not, apply to fxSetup global:
                window.fxSetup = {
                    from: base,
                    to: actualCurrency,
                    rates: currencyRates,
                    base: base
                };
            }
            updated = true;
            ionic.EventController.trigger('updateCurrencyRatesFinished', true);
        }, function () {
            ionic.EventController.trigger('updateCurrencyRatesFinished', false);
        });
        */
    };
})

// Location Provider Service
.service('locationProviderService', ['$http', '$rootScope', 'dataSavingService', 'popUpService', function($http, $rootScope, dataSavingService, notificationService) {
    var location = {
        latitude: null,
        longitude: null,
        cityName: null,
        countryName: null,
        countryShort: null
    };

    var distanceOptions = {
        metric: {
            small: {
                name: 'meters',
                shortName: 'm',
                rate: 1
            },
            big: {
                name: 'kilometers',
                shortName: 'km',
                rate: 0.001
            }
        },
        imperial: {
            small: {
                name: 'feet',
                shortName: 'ft',
                rate: 3.2808399
            },
            big: {
                name: 'miles',
                shortName: 'miles',
                rate: 0.000621371192
            }
        }
    };
    var distanceUnit = null;

    /**
     * Returns the current distance unit
     * @return {Object}
     */
    this.getDistanceUnit = function() {
        return distanceUnit;
    };

    /**
     * Sets a new distance unit to the service
     * @param {String} newOption
     */
    this.setDistanceUnit = function(newOption) {
        if (distanceOptions.hasOwnProperty(newOption)) {
            dataSavingService.saveItem('distanceUnit', newOption);
            distanceUnit = distanceOptions[newOption];
            return true;
        } else {
            return false;
        }
    };

    /**
     * Updates the user's location using Google Maps Geolocation API
     * Requests the location details Google Maps Geocoding API and broadcasts an event when the location is defined
     */
    this.updateLocation = function() {
        var APIKey = "AIzaSyBdNvbSInjgS9wQtj_ZHFP1bzP2NRm8gzY";

        navigator.geolocation.getCurrentPosition(function(position) {
            location.latitude  = position.coords.latitude;
            location.longitude = position.coords.longitude;
            var url = "https://maps.googleapis.com/maps/api/geocode/json?latlng=" + location.latitude + ", " + location.longitude + "&sensor=true&language=en&key=" + APIKey;

            if(location.latitude != null && location.longitude != null) {
                $http({
                    method: 'GET',
                    url: url,
                    cache: false
                }).then(function(response) {
                    for(var i in response.data.results) {
                        if(response.data.results[i].types[0] == "locality") {
                            for(var j in response.data.results[i].address_components) {
                                // Getting the country
                                if(response.data.results[i].address_components[j].types[0] == "country") {
                                    location.countryShort = response.data.results[i].address_components[j].short_name;
                                    location.countryName = response.data.results[i].address_components[j].long_name;
                                }
                                // Getting the city
                                if(response.data.results[i].address_components[j].types[0] == "locality") {
                                    location.cityName = response.data.results[i].address_components[j].long_name;
                                }
                            }
                        }
                    }

                    ionic.EventController.trigger('locationDefined');

                }, function() {
                    // Geocode connection error
                });
            } else {
                // Geolocation response error
            }
        }, function() {
            // Geolocation connection error
        });
    }

    /**
     * Returns the user's location. Always call updateLocation() before calling this function!
     * @return {Object}
     */
    this.getLocation = function() {
        return location;
    };

    /**
     * Calculates the distance according to the current position and the given coordinates
     * @param  {double} lat1
     * @param  {double} lon1
     * @return {Object}
     */
    this.getDistance = function(lat1, lon1) {
        if(location.latitude == null || location.longitude == null) {
            return null;
        }
        var response = [];
        var radlat1 = Math.PI * lat1 / 180;
        var radlat2 = Math.PI * location.latitude / 180;
        var radlon1 = Math.PI * lon1 / 180;
        var radlon2 = Math.PI * location.longitude / 180;
        var theta = lon1 - location.longitude;
        var radtheta = Math.PI * theta / 180;
        var dist = Math.sin(radlat1) * Math.sin(radlat2) + Math.cos(radlat1) * Math.cos(radlat2) * Math.cos(radtheta);
        dist = Math.acos(dist);
        dist = dist * 180 / Math.PI * 60 * 1.1515 * 1.609344 * 1000;

        if (dist > 1000) {
            dist = dist * distanceUnit.big.rate;
            response['value'] = dist;
            response['unit'] = distanceUnit.big.shortName;
            response['distanceFractions'] = 2;
        } else {
            response['value'] = dist;
            response['unit'] = distanceUnit.small.shortName;
            response['distanceFractions'] = 0;
        }

        return response;
    };
}]);
